import georeference
import uncertainty
import math

#TODO formalize unit tests

latitude = math.radians(48.4525) 
longitude = math.radians(-68.5232)
ellipsoidal_height = -28
heading = math.radians(350)
pitch = math.radians(0.5)
roll = math.radians(1.5)
depth = 5
offset_x = 1
offset_y = 2
offset_z = 3
latitude_sigma = 0.00001
longitude_sigma = 0.0001
ellipsoidal_height_sigma = 1
heading_sigma = 1
pitch_sigma = 1
roll_sigma = 1
depth_sigma = 0.5
offset_x_sigma = 0.0000001
offset_y_sigma = 0.0000001
offset_z_sigma = 0.0000001


print("Georeferencing")
g = georeference.Georeference()

print("Raw position:")
print(g.llh2ecef(latitude,longitude,ellipsoidal_height))

print("Sounding position:")
print(g.compute(latitude,longitude,ellipsoidal_height,heading,pitch,roll,depth,offset_x,offset_y,offset_z))

print("Uncertainty")
u = uncertainty.TotalPropagatedUncertainty()

tpu = u.compute(latitude,longitude,ellipsoidal_height,heading,pitch,roll,depth,offset_x,offset_y,offset_z,latitude_sigma,longitude_sigma,ellipsoidal_height_sigma,heading_sigma,pitch_sigma,roll_sigma,depth_sigma,offset_x_sigma,offset_y_sigma,offset_z_sigma)



import math
import numpy as np

class Georeference:
	# Convert WGS84 lat/lon to ECEF
	def llh2ecef(self,latitude,longitude,ellipsoidal_height):
		return np.array([
			(6378137.0 / (math.sqrt(1 - (0.081819190842622 ** 2) * (math.sin(latitude) ** 2))) + ellipsoidal_height) * math.cos(latitude) * math.cos(longitude),
			(6378137.0 / (math.sqrt(1 - (0.081819190842622 ** 2) * (math.sin(latitude) ** 2))) + ellipsoidal_height) * math.cos(latitude) * math.sin(longitude),
			(6378137.0 / (math.sqrt(1 - (0.081819190842622 ** 2) * (math.sin(latitude) ** 2))) * (1 - (0.081819190842622 **2)) + ellipsoidal_height) * math.sin(latitude)
		])


	# Computes the position of a sonar sounding in the ECEF frame
	def compute(self,latitude,longitude,ellipsoidal_height,heading,pitch,roll,depth,offset_x,offset_y,offset_z):
		# Compute frame conversion matrix between local (NED) and terrestrial (ECEF) frames
		self.ned_2_ecef=np.array([
			[-math.cos(longitude)*math.sin(latitude),-math.sin(longitude),-math.cos(latitude)*math.cos(longitude)],
			[-math.sin(latitude)*math.sin(longitude),math.cos(longitude),-math.cos(latitude)*math.sin(longitude)],
			[math.cos(latitude),0,-math.sin(latitude)]
		])

		# Compute motion compensation matrix
		self.imu_2_ned=np.array([
			[math.cos(heading)*math.cos(pitch), math.cos(heading)*math.sin(pitch)*math.sin(roll)-math.cos(roll)*math.sin(heading), math.cos(heading)*math.cos(roll)*math.sin(pitch)+math.sin(roll)*math.sin(heading)],
			[math.cos(pitch)*math.sin(heading), math.cos(heading)*math.cos(roll)+math.sin(pitch)*math.sin(roll)*math.sin(heading), math.sin(heading)*math.cos(roll)*math.sin(pitch)-math.cos(heading)*math.sin(roll)],
			[-math.sin(pitch),  math.cos(pitch) * math.sin(roll), math.cos(roll) * math.cos(pitch)]
		]);

		# Sum the position, offset and depth vectors in the same frame (ECEF)
		return self.llh2ecef(latitude,longitude,ellipsoidal_height) + np.matmul(np.matmul(self.ned_2_ecef,self.imu_2_ned) , (np.array([offset_x,offset_y,offset_z]) + np.array([0,0,depth])));





import math
import numpy as np


class TotalPropagatedUncertainty:
	# Computes the total propagated error of the georeferencing equation found in georeference.py by multiplying its jacobian with the sensor covariance matrix
	def compute(self,latitude,longitude,ellipsoidalHeight,heading,pitch,roll,depth,x_offset,y_offset,z_offset,latitudeSigma,longitudeSigma,ellipsoidalHeightSigma,headingSigma,pitchSigma,rollSigma,depthSigma,xOffsetSigma,yOffsetSigma,zOffsetSigma):
		return np.array([
		[ (ellipsoidalHeightSigma**2)*(math.cos(latitude)**2)*(math.cos(longitude)**2) + ((math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - 			math.cos(heading)*math.sin(roll))*math.sin(longitude))**2)*(depthSigma**2) +
		((((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(longitude))*(depth + z_offset) + (math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(latitude) - math.cos(heading)*math.cos(pitch)*math.sin(longitude))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(longitude))*y_offset)**2)*(headingSigma**2) +
		(((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(longitude)*math.sin(latitude) - 42697.6727071805*(math.cos(latitude)**2)*math.cos(longitude)*math.sin(latitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) - 		(math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.cos(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.cos(longitude)*math.sin(latitude)*math.sin(pitch))*x_offset - (math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.cos(longitude))*y_offset)**2)*(latitudeSigma**2) +
		(((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude)*math.sin(longitude) - (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(depth + z_offset) - (math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.sin(heading) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch))*x_offset - (math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*y_offset)**2)*(longitudeSigma**2) +
		(((math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) + math.cos(pitch)*math.cos(roll)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.cos(roll)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(longitude)*math.sin(latitude)*math.sin(pitch) + math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.sin(heading)*math.sin(longitude)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) + math.cos(pitch)*math.sin(heading)*math.sin(longitude)*math.sin(roll) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)**2)*(pitchSigma**2) +
		(((math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*y_offset)**2)*(rollSigma**2) +
		((math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude) + math.cos(pitch)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch))**2)*(xOffsetSigma**2) +
		((math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))**2)*(yOffsetSigma**2) +
		((math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))**2)*(zOffsetSigma**2),


		(ellipsoidalHeightSigma**2)*(math.cos(latitude)**2)*math.cos(longitude)*math.sin(longitude) + (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(depthSigma**2) +
		(((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(longitude))*(depth + z_offset) + (math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(latitude) - math.cos(heading)*math.cos(pitch)*math.sin(longitude))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(longitude))*y_offset)*(((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude))*(depth + z_offset) + (math.cos(pitch)*math.sin(heading)*math.sin(latitude)*math.sin(longitude) + math.cos(heading)*math.cos(longitude)*math.cos(pitch))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude))*y_offset)*(headingSigma**2) +
		((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(longitude)*math.sin(latitude) - 42697.6727071805*(math.cos(latitude)**2)*math.cos(longitude)*math.sin(latitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.cos(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.cos(longitude)*math.sin(latitude)*math.sin(pitch))*x_offset - (math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.cos(longitude))*y_offset)*((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.sin(latitude)*math.sin(longitude) - 42697.6727071805*(math.cos(latitude)**2)*math.sin(latitude)*math.sin(longitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.sin(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(longitude) + math.sin(latitude)*math.sin(longitude)*math.sin(pitch))*x_offset - (math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.sin(longitude))*y_offset)*(latitudeSigma**2) -
		((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude)*math.cos(longitude) - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(depth + z_offset) - (math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude) + math.cos(pitch)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch))*x_offset - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*y_offset)*((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude)*math.sin(longitude) - (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(depth + z_offset) - (math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.sin(heading) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch))*x_offset - (math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*y_offset)*(longitudeSigma**2) +
		((math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) + math.cos(pitch)*math.cos(roll)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.cos(roll)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(longitude)*math.sin(latitude)*math.sin(pitch) + math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.sin(heading)*math.sin(longitude)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) + math.cos(pitch)*math.sin(heading)*math.sin(longitude)*math.sin(roll) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)*((math.cos(heading)*math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(heading) - math.cos(latitude)*math.cos(roll)*math.sin(longitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.sin(latitude)*math.sin(longitude)*math.sin(pitch) + math.cos(latitude)*math.cos(pitch)*math.sin(longitude) - math.cos(longitude)*math.sin(heading)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(roll) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)*(pitchSigma**2) +
		((math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*y_offset)*((math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*y_offset)*(rollSigma**2) +
		(math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude) + math.cos(pitch)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch))*(math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.sin(heading) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch))*(xOffsetSigma**2) +
		(math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(yOffsetSigma**2) +
		(math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(zOffsetSigma**2),


		(ellipsoidalHeightSigma**2)*math.cos(latitude)*math.cos(longitude)*math.sin(latitude) + (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*(depthSigma**2) -
		(x_offset*math.cos(latitude)*math.cos(pitch)*math.sin(heading) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*(depth + z_offset)*math.cos(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*y_offset*math.cos(latitude))*(((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(longitude))*(depth + z_offset) + (math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(latitude) - math.cos(heading)*math.cos(pitch)*math.sin(longitude))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(longitude))*y_offset)*(headingSigma**2) +
		((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(longitude)*math.sin(latitude) - 42697.6727071805*(math.cos(latitude)**2)*math.cos(longitude)*math.sin(latitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.cos(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.cos(longitude)*math.sin(latitude)*math.sin(pitch))*x_offset - (math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.cos(longitude))*y_offset)*((math.cos(latitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude))*(depth + z_offset) + (math.cos(heading)*math.cos(pitch)*math.sin(latitude) - math.cos(latitude)*math.sin(pitch))*x_offset + (math.cos(latitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude))*y_offset - (ellipsoidalHeight + 6.33543932729282e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude) - 42411.8382613839*math.cos(latitude)*(math.sin(latitude)**2)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)))*(latitudeSigma**2) -
		((math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) + math.cos(pitch)*math.cos(roll)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.cos(roll)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(longitude)*math.sin(latitude)*math.sin(pitch) + math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.sin(heading)*math.sin(longitude)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) + math.cos(pitch)*math.sin(heading)*math.sin(longitude)*math.sin(roll) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)*((math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.cos(roll) + math.cos(roll)*math.sin(latitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(latitude)*math.sin(pitch) - math.cos(pitch)*math.sin(latitude))*x_offset + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(roll) + math.sin(latitude)*math.sin(pitch)*math.sin(roll))*y_offset)*(pitchSigma**2) +
		((math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*y_offset)*((math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(depth + z_offset) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*y_offset)*(rollSigma**2) - 
		(math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude) + math.cos(pitch)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch))*(math.cos(heading)*math.cos(latitude)*math.cos(pitch) + math.sin(latitude)*math.sin(pitch))* (xOffsetSigma**2) +
		(math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(yOffsetSigma**2) +
		(math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(math.cos(pitch)*math.cos(roll)*math.sin(latitude) - 	(math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*(zOffsetSigma**2)
		],
		[
		(ellipsoidalHeightSigma**2)*(math.cos(latitude)**2)*math.cos(longitude)*math.sin(longitude) + (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(depthSigma**2) +
		(((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(longitude))*(depth + z_offset) + (math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(latitude) - math.cos(heading)*math.cos(pitch)*math.sin(longitude))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(longitude))*y_offset)*(((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude))*(depth + z_offset) + (math.cos(pitch)*math.sin(heading)*math.sin(latitude)*math.sin(longitude) + math.cos(heading)*math.cos(longitude)*math.cos(pitch))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude))*y_offset)*(headingSigma**2) +
		((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(longitude)*math.sin(latitude) - 42697.6727071805*(math.cos(latitude)**2)*math.cos(longitude)*math.sin(latitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.cos(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.cos(longitude)*math.sin(latitude)*math.sin(pitch))*x_offset - (math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.cos(longitude))*y_offset)*((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.sin(latitude)*math.sin(longitude) - 42697.6727071805*(math.cos(latitude)**2)*math.sin(latitude)*math.sin(longitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.sin(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(longitude) + math.sin(latitude)*math.sin(longitude)*math.sin(pitch))*x_offset - (math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.sin(longitude))*y_offset)*(latitudeSigma**2) -
		((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude)*math.cos(longitude) - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(depth + z_offset) - (math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude) + math.cos(pitch)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch))*x_offset - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*y_offset)*((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude)*math.sin(longitude) - (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(depth + z_offset) - (math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.sin(heading) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch))*x_offset - (math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*y_offset)*(longitudeSigma**2) +
		((math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) + math.cos(pitch)*math.cos(roll)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.cos(roll)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(longitude)*math.sin(latitude)*math.sin(pitch) + math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.sin(heading)*math.sin(longitude)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) + math.cos(pitch)*math.sin(heading)*math.sin(longitude)*math.sin(roll) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)*((math.cos(heading)*math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(heading) - math.cos(latitude)*math.cos(roll)*math.sin(longitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.sin(latitude)*math.sin(longitude)*math.sin(pitch) + math.cos(latitude)*math.cos(pitch)*math.sin(longitude) - math.cos(longitude)*math.sin(heading)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(roll) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)*(pitchSigma**2) +
		((math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*y_offset)*((math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*y_offset)*(rollSigma**2) +
		(math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude) + math.cos(pitch)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch))*(math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.sin(heading) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch))*(xOffsetSigma**2) +
		(math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(yOffsetSigma**2) +
		(math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - 	math.cos(heading)*math.sin(roll))*math.sin(longitude))*(math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(zOffsetSigma**2),
		(ellipsoidalHeightSigma**2)*(math.cos(latitude)**2)*(math.sin(longitude)**2) + ((math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - 	math.cos(heading)*math.sin(roll))*math.cos(longitude))**2)*(depthSigma**2) +
		((((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude))*(depth + z_offset) + (math.cos(pitch)*math.sin(heading)*math.sin(latitude)*math.sin(longitude) + math.cos(heading)*math.cos(longitude)*math.cos(pitch))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude))*y_offset)**2)*(headingSigma**2) +
		(((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.sin(latitude)*math.sin(longitude) - 42697.6727071805*(math.cos(latitude)**2)*math.sin(latitude)*math.sin(longitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.sin(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(longitude) + math.sin(latitude)*math.sin(longitude)*math.sin(pitch))*x_offset - (math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.sin(longitude))*y_offset)**2)*(latitudeSigma**2) +
		(((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude)*math.cos(longitude) - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(depth + z_offset) - (math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude) + math.cos(pitch)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch))*x_offset - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*y_offset)**2)*(longitudeSigma**2) +
		(((math.cos(heading)*math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(heading) - math.cos(latitude)*math.cos(roll)*math.sin(longitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.sin(latitude)*math.sin(longitude)*math.sin(pitch) + math.cos(latitude)*math.cos(pitch)*math.sin(longitude) - math.cos(longitude)*math.sin(heading)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(roll) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)**2)*(pitchSigma**2) +
		(((math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*y_offset)**2)*(rollSigma**2) +
		((math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.sin(heading) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch))**2)*(xOffsetSigma**2) +
		((math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))**2)*(yOffsetSigma**2) + 
		((math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))**2)*(zOffsetSigma**2),

		(ellipsoidalHeightSigma**2)*math.cos(latitude)*math.sin(latitude)*math.sin(longitude) + (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*(depthSigma**2) - 
		(x_offset*math.cos(latitude)*math.cos(pitch)*math.sin(heading) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*(depth + z_offset)*math.cos(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*y_offset*math.cos(latitude))*(((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude))*(depth + z_offset) + (math.cos(pitch)*math.sin(heading)*math.sin(latitude)*math.sin(longitude) + math.cos(heading)*math.cos(longitude)*math.cos(pitch))*x_offset + 		((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude))*y_offset)*(headingSigma**2) + 
		((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.sin(latitude)*math.sin(longitude) - 42697.6727071805*(math.cos(latitude)**2)*math.sin(latitude)*math.sin(longitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - 	(math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.sin(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(longitude) + math.sin(latitude)*math.sin(longitude)*math.sin(pitch))*x_offset - (math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - 	(math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.sin(longitude))*y_offset)*((math.cos(latitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude))*(depth + z_offset) + (math.cos(heading)*math.cos(pitch)*math.sin(latitude) - 	math.cos(latitude)*math.sin(pitch))*x_offset + (math.cos(latitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude))*y_offset - (ellipsoidalHeight + 6.33543932729282e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude) - 	42411.8382613839*math.cos(latitude)*(math.sin(latitude)**2)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)))*(latitudeSigma**2) -
		((math.cos(heading)*math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(heading) - math.cos(latitude)*math.cos(roll)*math.sin(longitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.sin(latitude)*math.sin(longitude)*math.sin(pitch) + 	math.cos(latitude)*math.cos(pitch)*math.sin(longitude) - math.cos(longitude)*math.sin(heading)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(roll) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)*((math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.cos(roll) + math.cos(roll)*math.sin(latitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(latitude)*math.sin(pitch) - math.cos(pitch)*math.sin(latitude))*x_offset + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(roll) + math.sin(latitude)*math.sin(pitch)*math.sin(roll))*y_offset)*(pitchSigma**2) +
		((math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*y_offset)*((math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(depth + z_offset) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*y_offset)*(rollSigma**2) -
		(math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.sin(heading) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch))*(math.cos(heading)*math.cos(latitude)*math.cos(pitch) + math.sin(latitude)*math.sin(pitch))*(xOffsetSigma**2) +
		(math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(yOffsetSigma**2) +
		(math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*(zOffsetSigma**2)
		],
		[ 
		(ellipsoidalHeightSigma**2)*math.cos(latitude)*math.cos(longitude)*math.sin(latitude) + (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*(depthSigma**2) -
		(x_offset*math.cos(latitude)*math.cos(pitch)*math.sin(heading) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*(depth + z_offset)*math.cos(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*y_offset*math.cos(latitude))*(((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(longitude))*(depth + z_offset) + (math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(latitude) - math.cos(heading)*math.cos(pitch)*math.sin(longitude))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude)*math.sin(latitude) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(longitude))*y_offset)* (headingSigma**2) +
		((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(longitude)*math.sin(latitude) - 42697.6727071805*(math.cos(latitude)**2)*math.cos(longitude)*math.sin(latitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.cos(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.cos(longitude)*math.sin(latitude)*math.sin(pitch))*x_offset - (math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.cos(longitude))*y_offset)*((math.cos(latitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude))*(depth + z_offset) + (math.cos(heading)*math.cos(pitch)*math.sin(latitude) - math.cos(latitude)*math.sin(pitch))*x_offset + (math.cos(latitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude))*y_offset - (ellipsoidalHeight + 6.33543932729282e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude) - 42411.8382613839*math.cos(latitude)*(math.sin(latitude)**2)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)))*(latitudeSigma**2) -
		((math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(latitude) + math.cos(pitch)*math.cos(roll)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.cos(roll)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(longitude)*math.sin(latitude)*math.sin(pitch) + math.cos(latitude)*math.cos(longitude)*math.cos(pitch) + math.sin(heading)*math.sin(longitude)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude)*math.sin(roll) + math.cos(pitch)*math.sin(heading)*math.sin(longitude)*math.sin(roll) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)*((math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.cos(roll) + math.cos(roll)*math.sin(latitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(latitude)*math.sin(pitch) - math.cos(pitch)*math.sin(latitude))*x_offset + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(roll) + math.sin(latitude)*math.sin(pitch)*math.sin(roll))*y_offset)*(pitchSigma**2) +
		((math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*y_offset)*((math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(depth + z_offset) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*y_offset)*(rollSigma**2) -
		(math.cos(heading)*math.cos(longitude)*math.cos(pitch)*math.sin(latitude) + math.cos(pitch)*math.sin(heading)*math.sin(longitude) - math.cos(latitude)*math.cos(longitude)*math.sin(pitch))*(math.cos(heading)*math.cos(latitude)*math.cos(pitch) + math.sin(latitude)*math.sin(pitch))*(xOffsetSigma**2) +
		(math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude)*math.sin(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(longitude))*(math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(yOffsetSigma**2) +
		(math.cos(latitude)*math.cos(longitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude)*math.sin(latitude) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(longitude))*(math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*(zOffsetSigma**2),  
		(ellipsoidalHeightSigma**2)*math.cos(latitude)*math.sin(latitude)*math.sin(longitude) + (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*(depthSigma**2) -
		(x_offset*math.cos(latitude)*math.cos(pitch)*math.sin(heading) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*(depth + z_offset)*math.cos(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*y_offset*math.cos(latitude))*(((math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(longitude))*(depth + z_offset) + (math.cos(pitch)*math.sin(heading)*math.sin(latitude)*math.sin(longitude) + math.cos(heading)*math.cos(longitude)*math.cos(pitch))*x_offset + ((math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.sin(latitude)*math.sin(longitude) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(longitude))*y_offset)*(headingSigma**2) +
		((ellipsoidalHeight + 6.37813700000000e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.sin(latitude)*math.sin(longitude) - 42697.6727071805*(math.cos(latitude)**2)*math.sin(latitude)*math.sin(longitude)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude)*math.sin(longitude))*(depth + z_offset) + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(longitude) + math.sin(latitude)*math.sin(longitude)*math.sin(pitch))*x_offset - (math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude)*math.sin(longitude))*y_offset)*((math.cos(latitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude))*(depth + z_offset) + (math.cos(heading)*math.cos(pitch)*math.sin(latitude) - math.cos(latitude)*math.sin(pitch))*x_offset + (math.cos(latitude)*math.cos(pitch)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude))*y_offset - (ellipsoidalHeight + 6.33543932729282e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude) - 42411.8382613839*math.cos(latitude)*(math.sin(latitude)**2)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)))*(latitudeSigma**2) -
		((math.cos(heading)*math.cos(pitch)*math.cos(roll)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.cos(roll)*math.sin(heading) - math.cos(latitude)*math.cos(roll)*math.sin(longitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.sin(latitude)*math.sin(longitude)*math.sin(pitch) + math.cos(latitude)*math.cos(pitch)*math.sin(longitude) - math.cos(longitude)*math.sin(heading)*math.sin(pitch))*x_offset + (math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude)*math.sin(roll) - math.cos(longitude)*math.cos(pitch)*math.sin(heading)*math.sin(roll) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch)*math.sin(roll))*y_offset)*((math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.cos(roll) + math.cos(roll)*math.sin(latitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(latitude)*math.sin(pitch) - math.cos(pitch)*math.sin(latitude))*x_offset + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(roll) + math.sin(latitude)*math.sin(pitch)*math.sin(roll))*y_offset)*(pitchSigma**2) +
		((math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(depth + z_offset) - (math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*y_offset)*((math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(depth + z_offset) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*y_offset)*(rollSigma**2) -
		(math.cos(heading)*math.cos(pitch)*math.sin(latitude)*math.sin(longitude) - math.cos(longitude)*math.cos(pitch)*math.sin(heading) - math.cos(latitude)*math.sin(longitude)*math.sin(pitch))*(math.cos(heading)*math.cos(latitude)*math.cos(pitch) + math.sin(latitude)*math.sin(pitch))*(xOffsetSigma**2) +
		(math.cos(latitude)*math.cos(pitch)*math.sin(longitude)*math.sin(roll) + (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude)*math.sin(longitude) - (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*math.cos(longitude))*(math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(yOffsetSigma**2) +
		(math.cos(latitude)*math.cos(pitch)*math.cos(roll)*math.sin(longitude) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude)*math.sin(longitude) - (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*math.cos(longitude))*(math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))*(zOffsetSigma**2),
		((math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))**2)*(depthSigma**2) +
		((x_offset*math.cos(latitude)*math.cos(pitch)*math.sin(heading) + (math.cos(roll)*math.sin(heading)*math.sin(pitch) - math.cos(heading)*math.sin(roll))*(depth + z_offset)*math.cos(latitude) + (math.sin(heading)*math.sin(pitch)*math.sin(roll) + math.cos(heading)*math.cos(roll))*y_offset*math.cos(latitude))**2)*(headingSigma**2) +
		(((math.cos(latitude)*math.cos(pitch)*math.cos(roll) + (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.sin(latitude))*(depth + z_offset) + (math.cos(heading)*math.cos(pitch)*math.sin(latitude) - math.cos(latitude)*math.sin(pitch))*x_offset + (math.cos(latitude)*math.cos(pitch)*math.sin(roll) + 	(math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.sin(latitude))*y_offset - (ellipsoidalHeight + 6.33543932729282e6/math.sqrt(-0.00669437999014140*(math.sin(latitude)**2) + 1))*math.cos(latitude) - 42411.8382613839*math.cos(latitude)*(math.sin(latitude)**2)/((-0.00669437999014140*(math.sin(latitude)**2) + 1)**(3/2)))**2)*(latitudeSigma**2) +
		(((math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.cos(roll) + math.cos(roll)*math.sin(latitude)*math.sin(pitch))*(depth + z_offset) - (math.cos(heading)*math.cos(latitude)*math.sin(pitch) - math.cos(pitch)*math.sin(latitude))*x_offset + (math.cos(heading)*math.cos(latitude)*math.cos(pitch)*math.sin(roll) + 	math.sin(latitude)*math.sin(pitch)*math.sin(roll))*y_offset)**2)*(pitchSigma**2) +
		(((math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))*(depth + z_offset) - (math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + 	math.sin(heading)*math.sin(roll))*math.cos(latitude))*y_offset)**2)*(rollSigma**2) +
		((math.cos(heading)*math.cos(latitude)*math.cos(pitch) + math.sin(latitude)*math.sin(pitch))**2)*(xOffsetSigma**2) +
		((math.cos(pitch)*math.sin(latitude)*math.sin(roll) - (math.cos(heading)*math.sin(pitch)*math.sin(roll) - math.cos(roll)*math.sin(heading))*math.cos(latitude))**2)*(yOffsetSigma**2) +
		((math.cos(pitch)*math.cos(roll)*math.sin(latitude) - (math.cos(heading)*math.cos(roll)*math.sin(pitch) + math.sin(heading)*math.sin(roll))*math.cos(latitude))**2)*(zOffsetSigma**2) +
		(ellipsoidalHeightSigma**2)*(math.sin(latitude)**2)
		]
		])



from setuptools import setup, find_packages

setup(
    name="csbprivatetest",            # Name of your package
    version="0.1.0",                # Version number
    description="Crowdsourced bathymetry toolkit",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Vishwa Barathy",
    author_email="vishwa.barathy@cidco.ca",
    packages=find_packages(),       # Automatically find packages
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
